//Zong, by adam sepanski on 12/14/2006
//artwork by Jason Cook
//a simple exercise in XNA and my first program in C# and XNA.

#region Using Statements
using System;
using System.Collections.Generic;
using System.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Zong
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        SpriteBatch mSpriteBatch;
        KeyboardState mKeyboardState;

        Texture2D mBackground;

        Ball mBall;
        PlayerPaddle mPaddlePlayer;
        EnemyPaddle mPaddleEnemy;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            
            //not sure why i do this, try content if this works
            ContentManager aContentManager = new ContentManager(Services);

            mSpriteBatch = new SpriteBatch(this.graphics.GraphicsDevice);
            mBall = new Ball(this.graphics.GraphicsDevice, aContentManager , 60, 60);
            mPaddlePlayer = new PlayerPaddle(this.graphics.GraphicsDevice, aContentManager, 42, 154, 10, 200);
            mPaddleEnemy = new EnemyPaddle(this.graphics.GraphicsDevice, aContentManager, 42, 154, Window.ClientBounds.Width-42-10, 50);
            mBackground = aContentManager.Load<Texture2D>("./images/board_1024x768") as Texture2D;

            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            mKeyboardState = Keyboard.GetState();

            //move paddle
            mPaddlePlayer.Update(Window, mKeyboardState);
            mPaddleEnemy.Update(Window, mBall.Y + (mBall.Height / 2));
            //move ball
            mBall.Update(Window);
            //ball collisions with paddle
            BallCollisionsWithPaddle();

            //exit game if escape is pressed
            if (mKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            base.Update(gameTime);
        }

        public void BallCollisionsWithPaddle()
        {
            if (mBall.Y + mBall.Height > mPaddlePlayer.Y && mBall.Y < mPaddlePlayer.Y + mPaddlePlayer.Height)
            {
                if (mBall.X <= mPaddlePlayer.X + mPaddlePlayer.Width)
                {
                    mBall.X = mPaddlePlayer.X + mPaddlePlayer.Width;
                    mBall.VX *= -1;
                }
            }
            if (mBall.Y + mBall.Height > mPaddleEnemy.Y && mBall.Y < mPaddleEnemy.Y + mPaddleEnemy.Height)
            {
                if (mBall.X+mBall.Width>= mPaddleEnemy.X)
                {
                    mBall.X = mPaddleEnemy.X - mBall.Width;
                    mBall.VX *= -1;
                }
            }
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            //begin drawing the sprites
            mSpriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            //draw background scaled to window
            mSpriteBatch.Draw(mBackground, 
                new Rectangle(0,0,Window.ClientBounds.Width,Window.ClientBounds.Height),
                Color.White);

            //draw ball
            mBall.Draw(mSpriteBatch);
            //draw paddle
            mPaddlePlayer.Draw(mSpriteBatch);
            mPaddleEnemy.Draw(mSpriteBatch);

            //stop drawing the sprites
            mSpriteBatch.End();
        
            base.Draw(gameTime);
        }
    }
}